/* PyWX_buffer source
 *
 * Copyright (C) 1999 Brent A. Fulgham <bfulgham@debian.org>
 * http://www.debian.org
 *
 * See also http://www.aolserver.com for details on the AOLserver
 * web server software
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
 * MA 02139, USA.
 * 
 */

/*
 * The contents of this source file are based heavily on the following
 * sources:
 *
 * (*) Rob Mayoff's "tclpages" source code
 * (*) Lele Gaifax's "PyApache" source code
 * (*) Examples from the Python source distribution (especially the
 *     chapters on extended and embedding python)
 *
 * This source file is just the first step in Python world domination.
 * If this unit is successful, it will mean the begining of my secret
 * plot to replace the AOLserver Tcl base with Python.  Mwahahahaha.
 *
 * By the way, I tend to comment verbosely, so sorry if you are printing
 * this thing out.  But if I don't make notations here, I forget why I
 * did things and that just makes life harder in general.  Besides,
 * you're not paying for this so just enjoy. :-)
 * 
 */

#include "PyWX_buffer.hh"

//
//  Documentation strings
//
#define FUNDESCR(s) s "<BR>"
#define FUNSYN(s)  "Synopsis:\t" s "<BR>"
#define FUNARGS(s) "Arguments:\t" s "<BR>"
#define FUNRET(s)  "Returns:\t" s "<BR>"
#define FUNDOC(d,s,a,r)  FUNDESCR(d) FUNSYN(s) FUNARGS(a) FUNRET(r)

char PyWX_buffer::_doc[] =
    "PyWX_buffer is a Python Buffer object for I/O to AOLserver";
char PyWX_buffer::read_doc[] =
    FUNDOC("Read data from the client", ".read (size)",
           "an optional count of bytes to read, like standard file objects",
           "a string object");
char PyWX_buffer::readline_doc[] =
    FUNDOC("Read an entire line from the client",
           ".readline (size)",
           "an optional count of chars to read, like "
               "standard .readline() methods",
           "a string object");
char PyWX_buffer::readlines_doc[] =
    FUNDOC("Read an entire file from the client",
           ".readlines (size)",
           "an optional count of chars to read, like "
               "standard .readlines() methods",
           "a list of strings");
char PyWX_buffer::tell_doc[] =
    FUNDOC("Return the current position in the input stream",
           ".tell ()", "none", "an integer value");
char PyWX_buffer::write_doc[] =
    FUNDOC("Write a string to the client",
           ".write (string)", "the string to be written", "None");
char PyWX_buffer::flush_doc[] =
    FUNDOC("No-op method, for compatibility", ".flush()", "none", "None");
char PyWX_buffer::close_doc[] =
    FUNDOC("No-op method, for compatibility", ".close()", "none", "None");


extern "C"
{
    static PyMethodDef methods[];
    static PyMethodDef instMethods[];
    static memberlist members[];

    static PyObject *New(Ns_Conn *connection);
    static void Destroy(PyWX_buffer *object);
    static PyObject *getAttribute(PyWX_buffer *object, char *name);
    static int setAttribute(PyWX_buffer *object, char *name, PyObject *value);
    static PyObject *Read(PyWX_buffer *object, PyObject *args);
    static PyObject *Readline(PyWX_buffer *object, PyObject *args);
    static PyObject *Readlines(PyWX_buffer *object, PyObject *args);
    static int Tell(PyWX_buffer *object);
    static PyObject *Close(PyWX_buffer *object, PyObject *args);
    static PyObject *Flush(PyWX_buffer *object, PyObject *args);
    static PyObject *Write(PyWX_buffer *object, PyObject *args);
}


// The following definitions are trivial wrappers around PyWX_buffer
// method functions so that they can be used as Python callback
// functions.

PyObject *getAttribute(PyWX_buffer *object, char *name)
{ return object->getattr(name); }

int setAttribute(PyWX_buffer *object, char *name, PyObject *value)
{ return object->setattr(name, value); }

PyObject *Read(PyWX_buffer *object, PyObject *args)
{ return object->read(args); }

PyObject *Readline(PyWX_buffer *object, PyObject *args)
{ return object->readline(args); }

PyObject *Readlines(PyWX_buffer *object, PyObject *args)
{ return object->readlines(args); }

int Tell(PyWX_buffer *object)
{ return object->internBuff->tell(); }

PyObject *Close(PyWX_buffer *object, PyObject *args)
{ return object->close(args); }

PyObject *Flush(PyWX_buffer *object, PyObject *args)
{ return object->flush(args); }

PyObject *Write(PyWX_buffer *object, PyObject *args)
{ return object->write(args); }


PyObject *New(Ns_Conn *connection)
{
    Ns_Log(Debug, "In static New");
    PyWX_buffer *obj = PyObject_NEW(PyWX_buffer, &PyWXbuffType);
    if (obj == NULL)
	return NULL;
    obj->Init(obj, connection);
    return (PyObject *) obj;
}


void Destroy(PyWX_buffer *object)
{
    Ns_Log(Debug, "In static Destroy");
    // Destroy in place without freeing memory.  @@MRH is this what is wanted?
    object->~PyWX_buffer();
}


// These are "instance" methods -- not used at the "class" level
PyMethodDef
instMethods[] = {
    {"read", (PyCFunction) Read, METH_VARARGS, PyWX_buffer::read_doc},
    {"readline", (PyCFunction) Readline, METH_VARARGS,
     PyWX_buffer::readline_doc},
    {"readlines", (PyCFunction) Readlines, METH_VARARGS,
     PyWX_buffer::readlines_doc},
    {"tell", (PyCFunction) Tell, METH_VARARGS, PyWX_buffer::tell_doc},
    {"write", (PyCFunction) Write, METH_VARARGS, PyWX_buffer::write_doc},
    {"flush", (PyCFunction) Flush, METH_VARARGS, PyWX_buffer::flush_doc},
    {"close", (PyCFunction) Close, METH_VARARGS, PyWX_buffer::close_doc},
    {0, 0, 0, 0},
};

PyMethodDef
methods[] = {
    {"NewBuff", (PyCFunction) New, METH_VARARGS},
    {0, 0, 0, 0},
};

memberlist
members[] = {
    {"softspace", T_INT, offsetof(PyWX_buffer, f_softspace)},
    {0, 0, 0, 0}
};


/*
 *  Python initializer method.  It's declared extern "C".
 */
void
initPyWXbuffer()
{
    Py_InitModule("PyWX_buffer", methods);
}


PyWX_buffer::PyWX_buffer()
{
    Init(this, NULL);
}


PyWX_buffer::~PyWX_buffer()
{
    internBuff->sync();
    delete internBuff;
}


// Flush method
PyObject *PyWX_buffer::flush(PyObject *args)
{
    if (! PyArg_ParseTuple(args, "")) {
        return NULL;
    }
    internBuff->flushBuffer();
    Py_INCREF(Py_None);
    return Py_None;
}


// Read a single line
PyObject *PyWX_buffer::readline(PyObject *args)
{
    long n = -1;
    if (! PyArg_ParseTuple(args, "|l;SIZE", &n)) {
        return NULL;
    }

    if (n == 0) {
        return (PyString_FromString(""));
    }

    if (n < 0) {
        n = 0;
    }

    return getline(n);
}


PyObject *PyWX_buffer::close(PyObject *args)
{
    if (! PyArg_ParseTuple(args, "")) {
        return NULL;
    }
    internBuff->sync();		// Make sure everything's been written
    Py_INCREF(Py_None);
    return Py_None;
}


//-----------------------------------------------------------------------------
//
//  void Init(PyWX_buffer* self, Ns_Conn* connection)
//
//  Python object initialization
//
//-----------------------------------------------------------------------------
void
PyWX_buffer::Init(PyWX_buffer *self, Ns_Conn *connection)
{
    Ns_Log(Debug, "PyWX_buffer: Initializing values");
    f_softspace = 0;
    internBuff = new AOLbuffer(connection);
    internBuff->Init();
}



//-----------------------------------------------------------------------------
//
//  PyObject* read(PyObject* args)
//
//  The following routine implements the Python object methods for
//  read/write streams.
//
//-----------------------------------------------------------------------------
PyObject *
PyWX_buffer::read(PyObject * args)
{
    Ns_Log(Debug, "PyWX_buffer::read");
    long bytesRequested = -1;
    size_t bufferSize = 0;
    if (! PyArg_ParseTuple(args, "|l", &bytesRequested)) {
        return NULL;
    }

    if (bytesRequested < 0) {
        bufferSize = SMALLCHUNK;
    }
    else {
        bufferSize = bytesRequested;
    }

    Ns_Log(Debug, "PyWX_buffer::read: Asked for %d", bytesRequested);

    PyObject *v = PyString_FromStringAndSize((char *) NULL, bufferSize);
    if (v == NULL) {
        return NULL;
    }

    size_t bytesRead = 0;
    size_t chunksize = 0;
    while (1) {
        Py_BEGIN_ALLOW_THREADS
        chunksize = internBuff->sgetn(
                PyString_AS_STRING((PyStringObject *) v) + bytesRead,
                bufferSize - bytesRead
                );
        Py_END_ALLOW_THREADS
        if (chunksize == 0) {
            if (internBuff->EndOF()) {
                break;
            }

            PyErr_SetString(PyExc_IOError, "Cannot read from client.");
            Py_DECREF(v);
            return NULL;
        }
        bytesRead += chunksize;
        if (bytesRequested < 0) {
            bufferSize = bytesRead + SMALLCHUNK;
            if (_PyString_Resize(&v, bufferSize) < 0) {
                return NULL;
            }
        }
        else if (bytesRead >= (unsigned long) bytesRequested) {
            // Shouldn't exceed bytesRequested!
            assert(bytesRead == (unsigned long) bytesRequested);
            break;
        }
    }
    if (bytesRead != bufferSize) {
        _PyString_Resize(&v, bytesRead);
    }
    return v;
}


//-----------------------------------------------------------------------------
//
//  PyObject* getline(long size)
//
//  The following code is adapted from Python fileobject.c:1.5.1
//  (as modified for inclusion in PyApache)
//
//  Size argument interpretation:
//  >  0: max length
//  == 0: read arbitrary line
//  <  0: strip trialing '\n', raise EOFError if EOF reached immediately
//
//-----------------------------------------------------------------------------
PyObject *
PyWX_buffer::getline(long size)
{
    Ns_Log(Debug, "PyWX_buffer::getline");
    size_t rsize = ((size > 0) ? size : 100);
    PyObject *v = PyString_FromStringAndSize((char *) NULL, rsize);
    if (v == NULL) {
        return NULL;
    }

    char *buf = PyString_AS_STRING((PyStringObject *) v);
    char *end = buf + rsize;
    size_t total = 0;

    Py_BEGIN_ALLOW_THREADS
    while (1) {
        register int c = internBuff->sgetc();
        if (c == -1) {
            if (internBuff->ERROR()) {
                PyErr_SetString(PyExc_IOError, "Cannot read from client.");
                Py_DECREF(v);
                return NULL;
                // @@MRH Isn't it a problem that these error returns
                // bypass Py_END_ALLOW_THREADS?
            }

            /* EOF */
            Py_BLOCK_THREADS
            if (PyErr_CheckSignals()) {
                Py_DECREF(v);
                return NULL;
            }

            if ((size < 0)
                && (buf == PyString_AS_STRING((PyStringObject *) v))) {
                Py_DECREF(v);
                PyErr_SetString(PyExc_EOFError, "EOF when reading a line.");
                return NULL;
            }
            Py_UNBLOCK_THREADS
            break;
        }

        if ((*buf++ = c) == '\n') {
            if (size < 0) {
                buf--;
            }
            break;
        }

        if (buf == end) {
            if (size > 0) {
                break;
            }

            total = rsize;
            rsize += 1000;
            Py_BLOCK_THREADS
            if (_PyString_Resize(&v, rsize) < 0) {
                return NULL;
            }
            Py_UNBLOCK_THREADS
            buf = PyString_AS_STRING((PyStringObject *) v) + total;
            end = PyString_AS_STRING((PyStringObject *) v) + rsize;
        }
    }
    Py_END_ALLOW_THREADS
    total = buf - PyString_AS_STRING((PyStringObject *) v);
    if (total != rsize) {
        _PyString_Resize(&v, total);
    }
    return v;
}

//-----------------------------------------------------------------------------
//
//  PyObject* readlines(PyObject* args)
//
//  Python object readline method
//
//-----------------------------------------------------------------------------
PyObject *
PyWX_buffer::readlines(PyObject * args)
{
    Ns_Log(Debug, "PyWX_buffer::readlines");
    long sizehint = 0;
    if (! PyArg_ParseTuple(args, "|l", &sizehint)) {
        return NULL;
    }

    PyObject *list = PyList_New(0);
    PyObject *line = NULL;
    if (list == NULL) {
        return NULL;
    }

    size_t nread = 0;
    size_t nfilled = 0;
    size_t totalread = 0;
    size_t buffersize = SMALLCHUNK;
    char small_buffer[SMALLCHUNK];
    char *buffer = small_buffer;
    PyObject *big_buffer = NULL;

    while (1) {
        Py_BEGIN_ALLOW_THREADS
        nread = internBuff->sgetn(buffer + nfilled, buffersize - nfilled);
        Py_END_ALLOW_THREADS
        if (nread == 0) {
            sizehint = 0;
            if (internBuff->EndOF()) {
                break;
            }

            Py_DECREF(list);
            if (big_buffer) {
                Py_DECREF(big_buffer);
            }
            return NULL;
        }

        totalread += nread;
        char *p = (char *) memchr(buffer + nfilled, '\n', nread);
        if (p == NULL) {
            // Need a larger buffer to fit this line
            nfilled += nread;
            buffersize *= 2;
            if (big_buffer == NULL) {
                // Create a big buffer
                big_buffer = PyString_FromStringAndSize(NULL, buffersize);
                if (big_buffer == NULL) {
                    Py_DECREF(list);
                    return NULL;
                }
                buffer = PyString_AS_STRING(big_buffer);
                memcpy(buffer, small_buffer, nfilled);
            }
            else {
                // Grow the big buffer
                _PyString_Resize(&big_buffer, buffersize);
                buffer = PyString_AS_STRING(big_buffer);
            }
            continue;
        }

        char *end = buffer + nfilled + nread;
        char *q = buffer;

        do {
            // Process complete lines
            p++;
            line = PyString_FromStringAndSize(q, p - q);
            if (line == NULL) {
                Py_DECREF(list);
                if (big_buffer)
                    Py_DECREF(big_buffer);
                return NULL;
            }

            int err = PyList_Append(list, line);
            Py_DECREF(line);
            if (err != 0) {
                Py_DECREF(list);
                if (big_buffer)
                    Py_DECREF(big_buffer);
                return NULL;
            }

            q = p;
            p = (char *) memchr(q, '\n', end - q);
        } while (p != NULL);

        // Move the remaining incomplete line to the start
        nfilled = end - q;
        memmove(buffer, q, nfilled);
        if (sizehint > 0) {
            if (totalread >= ((size_t) sizehint)) {
                break;
            }
        }

        if (nfilled != 0) {
            // Partial last line
            line = PyString_FromStringAndSize(buffer, nfilled);
            if (line == NULL) {
                Py_DECREF(list);
                if (big_buffer)
                    Py_DECREF(big_buffer);
                return NULL;
            }
            if (sizehint > 0) {
                // Need to complete the last line
                PyObject *rest = this->getline(0);
                if (rest == NULL) {
                    Py_DECREF(line);
                    Py_DECREF(list);
                    if (big_buffer)
                        Py_DECREF(big_buffer);
                    return NULL;
                }

                PyString_Concat(&line, rest);
                Py_DECREF(rest);

                if (line == NULL) {
                    Py_DECREF(list);
                    if (big_buffer)
                        Py_DECREF(big_buffer);
                    return NULL;
                }
            }

            int err = PyList_Append(list, line);
            Py_DECREF(line);
            if (err != 0) {
                Py_DECREF(line);
                if (big_buffer)
                    Py_DECREF(big_buffer);
                return NULL;
            }
        }
    }
    if (big_buffer)
        Py_DECREF(big_buffer);

    return list;
}


//-----------------------------------------------------------------------------
//
//  PyObject* write(PyObject* args)
//
//  Python object write method (sends output to the HTTP socket)
//
//-----------------------------------------------------------------------------
PyObject *
PyWX_buffer::write(PyObject *args)
{
    Ns_Log(Debug, "PyWX_buffer::write");
    if (internBuff->Malformed()) {
        PyErr_SetString(PyExc_IOError, "Script generated malformed headers.");
        return NULL;
    }

    char *strObj = NULL;
    int size = 0;
    if (! PyArg_ParseTuple(args, "s#:PyWX_buffer.write", &strObj, &size)) {
        return NULL;
    }

    // copy string so we can modify it in place:
    char s[size + 1];
    memcpy(s, strObj, size + 1); // try to tolerate \0 in string.
    char *str = s;

    // Did we parse and send the headers?
    if (! internBuff->HeaderSent() && ! internBuff->IsNPH()) {
        int consumed = internBuff->ParseHeaders(str, size);
        if (consumed < 0) {
            PyErr_SetString(PyExc_IOError,
                            "Script generated malformed headers");
            return NULL;
        }

        size -= consumed;
        str += consumed;
    }

    if (size != 0) {
        int amtPut = 0;
        Py_BEGIN_ALLOW_THREADS
        amtPut = internBuff->sputn(str, size);
        Py_END_ALLOW_THREADS
        if (amtPut != size) {
            PyErr_SetFromErrno(PyExc_IOError);
            return NULL;
        }
    }

    Py_INCREF(Py_None);
    return Py_None;
}


//-----------------------------------------------------------------------------
//
//  PyObject* getattr(char* name)
//
//  Python object gettattr method (used for object introspection)
//
//-----------------------------------------------------------------------------
PyObject *
PyWX_buffer::getattr(char *name)
{
    if (!strcmp(name, "__doc__")) {
        return PyString_FromString(_doc);
    }

    return Py_FindMethod(instMethods, (PyObject *)this, name);
}


//-----------------------------------------------------------------------------
//
//  PyObject* setattr(char* name, PyObject* value)
//
//  Python object setattr method (used to control the object from Python)
//
//-----------------------------------------------------------------------------
int
PyWX_buffer::setattr(char *name, PyObject * value)
{
    if (value == NULL) {
        PyErr_SetString(PyExc_AttributeError,
                        "can't delete PyWX_buffer attributes.");
        return NS_ERROR;
    }

    return PyMember_Set((char *) this, members, name, value);
}


// Type object for Python introspection
PyTypeObject PyWXbuffType = {
    PyObject_HEAD_INIT(&PyType_Type)
    0,				/* ob_size */
    "PyWX_buffer",		/* tp_name */
    sizeof(PyWX_buffer),	/* tp_basicsize */
    0,				/* tp_itemsize */

    /* methods */
    (destructor) Destroy,	/* tp_dealloc */
    (printfunc) 0,		/* tp_print */
    (getattrfunc) getAttribute,	/* tp_getattr */
    (setattrfunc) setAttribute,	/* tp_setattr */
    (cmpfunc) 0,		/* tp_compare */
    (reprfunc) 0,		/* tp_repr */
    0,				/* tp_as_numer */
    0,				/* tp_as_sequence */
    0,				/* tp_as_mapping */
    (hashfunc) 0,		/* tp_hash */
    (ternaryfunc) 0,		/* tp_call */
    (reprfunc) 0,		/* tp_str */
    (getattrofunc) 0,		/* tp_gettattro */
    (setattrofunc) 0,		/* tp_setattro */

    /* Space for future expansion */
    0L,
    0L,

    PyWX_buffer::_doc		/* Documentation string */
};


/*
 * $Header: /cvsroot/PyWX/PyWX/PyWX_buffer.cc,v 1.14 2000/08/26 01:24:57 mhagger Exp $
 */
