// For emacs==> -*- mode: C++; indent-tabs-mode: t; tab-width: 8; -*-

// $Id: PyWX_buffer.hh,v 1.10 2000/08/05 04:07:42 mhagger Exp $

/* PyWX_buffer source
 *
 * Copyright (C) 1999 Brent A. Fulgham <bfulgham@debian.org>
 * http://www.debian.org
 *
 * See also http://www.aolserver.com for details on the AOLserver
 * web server software
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
 * MA 02139, USA.
 * 
 */

/*
 * The contents of this source file are based heavily on the following
 * sources:
 *
 * (*) Rob Mayoff's "tclpages" source code
 * (*) Lele Gaifax's "PyApache" source code
 * (*) Examples from the Python source distribution (especially the
 *     chapters on extended and embedding python)
 *
 * By the way, I tend to comment verbosely, so sorry if you are printing
 * this thing out.  But if I don't make notations here, I forget why I
 * did things and that just makes life harder in general.  Besides,
 * you're not paying for this so just enjoy. :-)
 * 
 */

#ifndef __PyWX_BUFFER__
#define __PyWX_BUFFER__

//#define Ns_Log(a,b) cerr << b << endl

/* 
 *==========================================================================
 *  This is a buffer class that allows us to glue AOLserver and Python
 *  together.  I can't take credit for the idea -- it's based on the
 *  ApacheBuff object from PyApache, though its implementation as a
 *  C++ class is my own.
 *
 *  Actually, I can't even claim all of the C++ as my own -- much of it
 *  is based on the work of the Python-C++ SIG, and specifically the
 *  CXX effort (and Gordon McMillan's SCXX classes).
 *==========================================================================
 */

#include <aolserver/ns.h>
#include <AOLbuffer.hh>

// Python header files
#include <Python.h>
#include <compile.h>
#include <eval.h>
#include <structmember.h>


extern "C"
{
    // Type object for Python introspection
    extern PyTypeObject PyWXbuffType;

    // Module initialization function.  Should only be called once.
    void initPyWXbuffer();
}



struct PyWX_buffer
{
public:
    PyObject_HEAD
    AOLbuffer *internBuff;
    int f_softspace;		// Flag used by Python's 'print' command

    // Main Methods
    void Init(PyWX_buffer * self, Ns_Conn * connection);

    bool IsNPH()
    { return internBuff->IsNPH(); }

    void IsNPH(bool val)
    { internBuff->IsNPH(val); }

    PyWX_buffer();
    ~PyWX_buffer();

    PyObject *read(PyObject *args);		// Buffer read method
    PyObject *write(PyObject *args);		// Buffer write method
    PyObject *flush(PyObject *args);		// Flush method

    PyObject *getattr(char *name);		// Get attributes
    int setattr(char *name, PyObject * value);	// Set attributes

    // Implementations to support Python's stream interface
    PyObject *getline(long size);		// get a line from the client
    PyObject *readlines(PyObject *args);	// Readlines from the client
    PyObject *readline(PyObject *args);		// Read a single line
    PyObject *close(PyObject *args);

    // Documentation strings for "getattr" calls
    static char _doc[];
    static char read_doc[];
    static char readline_doc[];
    static char readlines_doc[];
    static char tell_doc[];
    static char write_doc[];
    static char flush_doc[];
    static char close_doc[];
};


#endif // PyWX_BUFFER

/*
 * $Header: /cvsroot/PyWX/PyWX/PyWX_buffer.hh,v 1.10 2000/08/05 04:07:42 mhagger Exp $
 */
