// For emacs==> -*- mode: C++; indent-tabs-mode: t; tab-width: 8; -*-

// $Id: ns_pywx.hh,v 1.3 2000/07/30 19:44:08 mhagger Exp $

// A set of routines to connect AOLserver with Python.

/* ns_pywx.hh Source
 *
 * Copyright (C) 1999 Brent A. Fulgham <bfulgham@debian.org>
 * http://www.debian.org
 *
 * See also http://www.aolserver.com for details on the AOLserver
 * web server software
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
 * MA 02139, USA.
 * 
 */

#ifndef __NS_PYWX__
#define __NS_PYWX__

#include <aolserver/ns.h>
#include <Python.h>


// The interface to AOLserver.  No name-mangling here, please.
extern "C"
{
    int Ns_ModuleInit(char *hServer, char *hModule);
}


// A class that can be used to hold a PyObject pointer that is owned
// by the current process.  When an instance is deleted, the pointer
// is DECREFed.
//
// WARNING: Do not use this class to hold a pointer to a `borrowed'
// object, because borrowed objects should not be DECREFed!
class TempPyObject
{
public:
    TempPyObject(PyObject *Iptr) : ptr(Iptr) {}
    ~TempPyObject() { Py_XDECREF(ptr); }

    operator PyObject *() { return ptr; }

    // Offer explicit method for contexts where the casting is not
    // done implicitly:
    PyObject *get_ptr() const { return ptr; }

    void XDECREF() { Py_XDECREF(ptr); ptr = 0; }

private:
    PyObject *ptr;
};

// Class can be used to convert a C string into a Python string,
// DECREFing the python string on destruction.
class TempPyString : public TempPyObject
{
public:
    TempPyString(const char *Is) : TempPyObject(PyString_FromString(Is)) {}
};


#endif // __NS_PYWX__

