# $Id: ns_setup.py,v 1.11.4.2 2000/08/29 14:18:48 mhagger Exp $

import os, string
import Ns

PyWX_Version = '0.6' # also set in ns_pywx.cc


def setup_global_interpreter():
    # Get the path to the Page Root:
    pythonPagePath = \
            Ns.ConfigGetValue('ns/environment/python', 'PageRoot')
    Ns.Log(Ns.Debug, 'PyWX: Python\'s path includes %s' % (pythonPagePath,))

    # Extract the current environment dictionary from the OS module,
    # make it empty, and then fill it with our variables.  In this
    # way, we can keep 'os.environ' and 'posix.environ' in sync --
    # since they are actually the same object, inserting a new dict in
    # one of the two modules would break the link.

    env = os.environ # nickname

    Ns.Log(Ns.Debug, 'PyWX: Clearing os.env')
    path = env['PATH'] # save path, though
    env.clear()
    env['PATH'] = path
    # The default search path now is whatever AOLserver set it to.
    Ns.Log(Ns.Debug, 'PyWX: os.environ[\'PATH\'] unchanged (%s)' % (path,))

    # Move the interpreter's execution root to the page root.
    os.chdir(pythonPagePath)

    # Set some variables in the namespace for potential script use
    env['PYWX_VERSION'] = PyWX_Version
    env['DOCUMENT_ROOT'] = pythonPagePath

    # Include info about our PyWX module
    env['SERVER_SOFTWARE'] = '%s/%s-PyWX/%s' % (
        Ns.InfoServerName(),
        Ns.InfoServerVersion(),
        PyWX_Version,
        )

    # Server Build date
    env['SERVER_BUILDDATE'] = Ns.InfoBuildDate()


def setup_env_strings():
    conn = Ns.GetConn()

    env = os.environ

    # Script Name:
    try:
        env['SCRIPT_NAME'] = Ns.UrlToFile(conn.Server(), conn.request.url)
    except RuntimeError:
        env['SCRIPT_NAME'] = ''

    # Set the HTTP_* header environment variables:
    headers = conn.Headers()
    for i in range(headers.Size()):
        key = string.replace(headers.Key(i), '-', '_')
        key = 'HTTP_' + string.upper(key)
        env[key] = headers.Value(i)

    # Connection Protocol:
    env['SERVER_PROTOCOL'] = 'HTTP/%4.2f' % (conn.request.version,)
    env['AUTH_TYPE'] = 'Basic'
    env['GATEWAY_INTERFACE'] = 'CGI/1.1'

    # Content Length/Content Type:
    content = headers.IGet('Content-type')
    if content is None:
        if conn.request.method == 'POST':
            content = 'application/x-www-form-urlencoded'
        else:
            content = ''
    env['CONTENT_TYPE'] = content

    if conn.ContentLength() <= 0:
        env['CONTENT_LENGTH'] = ''
    else:
        env['CONTENT_LENGTH'] = '%u' % (conn.ContentLength(),)

    env['REQUEST_METHOD'] = conn.request.method
    env['QUERY_STRING'] = conn.request.query or ''

    # Remote Address:
    connData = conn.Peer()
    if connData is not None:
        env['REMOTE_ADDR'] = connData
        env['REMOTE_HOST'] = Ns.GetHostByAddr(connData) or connData

    env['SERVER_NAME'] = conn.Host()

    # Server port (from AOLserver nscgi module logic)
    connData = conn.Location()
    port = None
    if connData is not None:
        try:
            port = Ns.ParseUrl(connData)[2]
        except RuntimeError:
            pass
    # If no port found, use default of '80':
    env['SERVER_PORT'] = port or '80'

    # Remote User
    if conn.authUser is not None:
        env['REMOTE_USER'] = conn.authUser

    # Unimplemented values:
    env['PATH_INFO'] = ''
    env['PATH_TRANSLATED'] = ''


# @MRH Is it a problem that multiple threads share the same sys.argv?
# It seems that multiple threads share sys.argv but multiple
# interpreters do not.
# @MRH Shouldn't this code do unquoting of the query string?
def setup_cmd_ln_args(filename):
    """Break up the query string and use it to set sys.argv."""

    import sys

    sys.argv = [filename]

    args = Ns.GetConn().request.query
  
    # If there are no parameters in the query, and this is not a FORM
    # submission, do not set any further arguments.
    if not args:
        return

    # The query info is split into separate arguments, where '+' is the
    # separator between keyword arguments.  We don't need to worry
    # about the case of zero args because it was handled above.
    sys.argv[1:] = string.split(args, '+')

    for i in range(len(sys.argv)):
        Ns.Log(Ns.Debug, "PyWX: Argument: %d is %s" % (i, sys.argv[i],))


def setup_cgi_env(filename):
    setup_env_strings()
    setup_cmd_ln_args(filename)


