# $Id: pywx.py,v 1.19.4.2 2000/08/29 14:29:35 mhagger Exp $

###
### See included file COPYRIGHT for copyright and license information.
###

# Should we use a custom version of execfile() that caches the
# compiled code in memory?
UseCachingExec = 0

# Should we install our version of execfile() for use by the scripts,
# too?
InstallCachingExec = 0

import sys, traceback, os, string, stat
import Ns

Ns.Log(Ns.Debug, '**** importing pywx.py')


if UseCachingExec:
    Ns.Log(Ns.Debug, 'Using the caching version of execfile.')
    import ns_codecache
    execfile = ns_codecache.execfile
else:
    Ns.Log(Ns.Debug, 'Using the standard (non-caching) version of execfile.')


if InstallCachingExec:
    Ns.Log(Ns.Debug, 'Will install caching version of execfile for scripts.')
    import ns_codecache # no harm if it's imported twice...


exception_html = """\
<html>
<title>PyWX: error</title>
<body bgcolor=white>
<h2>Error!</h2>
Unhandled exception returned from PyWX:

Can't run %(url)s --
<p>Traceback:
<blockquote><pre>
%(exc_text)s
</pre></blockquote>
</body>
</html>
"""

def runscript(script, url):
    """Run a python script, catching any exceptions and reporting them."""

    try:
        execfile(script, sys.modules['__main__'].__dict__)
    except SystemExit:
        # normal exit() call--needn't print exception.
        return
    except:
        # Don't store sys.exc_info() to local variables because it
        # causes a reference loop.
        exc_text = string.join(
            apply(traceback.format_exception, sys.exc_info()), '')
        Ns.GetConn().ReturnHtml(500, exception_html % locals())


def prepare_and_runscript(script, url):
    cwd = os.getcwd()

    scriptdir = os.path.dirname(script)
    os.chdir(scriptdir)

    # Add script's directory to sys.path:
    filedir = os.path.dirname(script)
    if filedir not in sys.path:
        sys.path.append(filedir)

    if InstallCachingExec:
        ns_codecache.initializeCachingExecFile()

    runscript(script, url)

    if InstallCachingExec:
        ns_codecache.uninitializeCachingExecFile()

    os.chdir(cwd)


def fulfill_request():
    conn = Ns.GetConn()
    url = conn.request.url
    script = Ns.UrlToFile(conn.Server(), url)
    Ns.Log(Ns.Notice, '++++ start python: ' + script)
    try:
        # see if file exists:
        os.stat(script)
    except OSError:
        conn.ReturnNotFound()
    else:
        # let python do its own error handling
        prepare_and_runscript(script, url)

    Ns.Log(Ns.Notice, '++++ done python: ' + script)


